@@ -26,6 +26,10 @@ namespace rfb {
   const int msgTypeBell = 2;
   const int msgTypeServerCutText = 3;
 
+  const int msgTypeEndOfContinuousUpdates = 150;
+
+  const int msgTypeServerFence = 248;
+
   // client to server
 
   const int msgTypeSetPixelFormat = 0;
@@ -36,6 +40,10 @@ namespace rfb {
   const int msgTypePointerEvent = 5;
   const int msgTypeClientCutText = 6;
 
+  const int msgTypeEnableContinuousUpdates = 150;
+
+  const int msgTypeClientFence = 248;
+
   const int msgTypeSetDesktopSize = 251;
 }
 #endif
